#include "experiment.hpp"
#include "model.hpp"
#include <eosim/dist/numbergenerator.hpp>
#include <cassert>

using namespace eosim::core;
using namespace eosim::dist;


Experiment::Experiment (): running(0), distMan(MT19937), currModel(0) {}

Experiment::~Experiment() {}

void Experiment::run (double simTime_) {
    assert(currModel != 0 && "Experiment::run: Error running a simulation");
    running = 1;
    bCal.setEndTime(simTime_);
    for (; (running); running = bCal.isStopped()) {
        bCal.bPhase();
        cCal.cPhase();
    }
    running = 0;
}

void Experiment::setModel (Model* model_) {
    assert(model_ != 0 && "Experiment::setModel: Error setting a Model");
    currModel = model_;
    setSeed(251182);
}

void Experiment::schedule (double offset_, Entity* who_) {
//    assert(running && "Experiment::schedule: Error scheduling an entity");
    bCal.schedule(offset_, who_);
}

void Experiment::setSeed (unsigned long seed_) {
    assert(currModel != 0 && "Experiment::setSeed: Error assigning a new seed to the experiment");
    distMan.setSeed(seed_);
}

void Experiment::registerDist (Distribution* dist_) {
    assert(currModel != 0 && "Experiment::registerDist: Error registering a Distribution");
    distMan.registerDist(dist_);
}

void Experiment::registerCEvent (CEvent* cEv_) {
    assert(currModel != 0 && "Experiment::registerCEvent: Error registering a CEvent");
    cCal.registerCEvent(cEv_);
}

double Experiment::getSimTime() {
    return bCal.getSimTime();
}

