#include "model.hpp"
#include <eosim/statics/histogram.hpp>
#include "bevent.hpp"
#include "cevent.hpp"
#include "experiment.hpp"
#include <eosim/dist/distribution.hpp>
#include <cassert>

using namespace eosim::core;
using namespace eosim::dist;
using namespace eosim::utils;
using namespace eosim::statics;


Model::Model(): exp(0) {}

Model::~Model() {}

void Model::connectToExp (Experiment *exp_) {
    assert(exp_ != 0 && "Model::connectToExp: Error connecting the model to an experiment");
    exp = exp_;
    exp->setModel(this);
    init();
	doInitialSchedules();
}

void Model::registerBEvent (BEvent* bEv_) {
    assert(exp != 0 && bEv_ != 0 && "Model::registerBEvent: Error registering a BEvent");
    bEvs.put(bEv_);
}

void Model::registerCEvent (CEvent *cEv_) {
    assert(exp != 0 && cEv_ != 0 && "Model::registerCEvent: Error registering a CEvent");
    exp->registerCEvent(cEv_);
}

void Model::registerDist (Distribution* dist_) {
    assert(exp != 0 && dist_ != 0 && "Model::registerDist: Error registering a Distribution");
    exp->registerDist(dist_);
}

void Model::registerHistogram (Histogram* hist_) {
    assert(exp != 0 && hist_ != 0 && "Model::registerHistogram: Error registering an Histogram");
    hist_->setModel(this);
}

void Model::schedule (double offset_, Entity* who_, std::string what_) {
    assert(exp != 0 && who_ != 0 && offset_ >= 0 && "Model::schedule: Error scheduling an Entity");
    BEvent* b = bEvs[what_];
    who_->setBEvent(b);
    exp->schedule (offset_, who_);
}

double Model::getSimTime() {
    assert(exp != 0 && "Model::getSimTime: Error getting the actual simulation time");
    return exp->getSimTime();
}

