#include "entityqueueordered.hpp"
#include <cassert>

using namespace eosim::core;
using namespace eosim::utils;
using namespace std;


EntityQueueOrdered::EntityQueueOrdered(EntityComp* comp_): comp(comp_) {}

EntityQueueOrdered::~EntityQueueOrdered() {
    for (deque<Entity*>::iterator i = elems.begin(); i != elems.end(); i++) {
        delete *i;
    }
}

void EntityQueueOrdered::push(Entity* ent_) {
    if (elems.empty() || comp->compare(ent_, elems[0]))
        elems.push_front(ent_);
    else if (comp->compare(elems[elems.size()-1], ent_))
        elems.push_back(ent_);
    else {
        unsigned int i = 0;
        unsigned int j = elems.size();
        for (; i != j - 1; ) {
            if (comp->compare(ent_, elems[(i+j)/2]))
                j = (i + j) / 2;
            else
                i = (i + j) / 2;
        }
        deque<Entity*>::iterator it = elems.begin();
        it += i+
        1;
        elems.insert(it, ent_);
    }
}

Entity* EntityQueueOrdered::pop() {
    assert(elems.size() > 0 && "EntityQueueOrdered::pop: Error removing the first element of a EntityQueueOrdered");
    Entity* e = elems.front();
    elems.pop_front();
    return e;
} 

void EntityQueueOrdered::remove(unsigned int i_) {
    assert(i_ < elems.size() && "EntityQueueOrdered::remove: Error removing an Entity from an EntityQueueOrdered");
    elems.erase(elems.begin() + i_);
}

bool EntityQueueOrdered::empty() {
    return elems.empty();
}

Entity* EntityQueueOrdered::operator[] (unsigned int i_) {
    assert(i_ < elems.size() && "EntityQueueOrdered::operator[]: Error indexing in an EntityQueueOrdered");
    return elems[i_];
}

unsigned int EntityQueueOrdered::size() {
    return elems.size();
}

